import React, { useState, useEffect } from 'react'
import { Button, message, TreeSelect, Table, Popconfirm } from 'antd'
import type { FormInstance } from 'antd'
import ProForm, { ModalForm, ProFormText } from '@ant-design/pro-form'
import type { ProFormColumnsType } from '@ant-design/pro-form'
import { EditableProTable } from '@ant-design/pro-table'
import ShowTitleMenu from '../Attendance/components/ShowTitleMenu'
import { useModel, useRequest } from 'umi'
import {
  addBusinessgroup,
  changeBusinessGroupStatus,
  delBusinessGroupById,
  getBusinessgroupList,
  updateBusinessGroup,
  getClientSourceList,
  addClientSource,
  updateClientSource
} from '@/services/user/api'
import type { ColumnsType } from 'antd/lib/table'
import { Delete } from '@icon-park/react'
import { useRef } from 'react'
import { PageContainer } from '@ant-design/pro-layout'

enum modalType {
  CREATE = 0,
  UPDATE = 1
}

const modalTitleType = {
  1: '商机角色',
  2: '来源'
}

interface StageSettingsType {
  id: React.Key
  name?: string
  stage?: string
  isDefault?: boolean
  percentage?: number
}

const defaultData = [
  {
    id: 624748504,
    stage: '结束',
    name: '赢单',
    isDefault: true,
    percentage: '100'
  },
  {
    id: 624691229,
    stage: '结束',
    name: '输单',
    isDefault: true,
    percentage: '0'
  },
  {
    id: 624691258,
    stage: '结束',
    name: '无效',
    isDefault: true,
    percentage: '0'
  }
]

const Contact: React.FC = () => {
  const { department, fetchDepartment } = useModel('department')
  const formRef = useRef<FormInstance>(null)

  const [editableKeys, setEditableRowKeys] = useState<React.Key[]>()

  const [state, setState] = useState({
    id: '',
    menuId: 1,
    dataSource: [],
    clientSourceList: [],
    visible: false,
    type: modalType.CREATE
  })
  // console.log(state.clientSourceList)

  const onSelect = (menuId: string) => {
    setState({ ...state, menuId })
  }

  const columns: ProFormColumnsType<StageSettingsType>[] = [
    {
      title: '状态',
      dataIndex: 'stage',
      width: '15%',
      // 第二行不允许编辑
      editable: false
    },
    {
      title: '状态名称',
      dataIndex: 'name',
      width: '28%'
    },
    {
      title: '赢单率',
      dataIndex: 'percentage',
      width: '27%',
      valueType: 'text',
      fieldProps: {
        addonAfter: '%'
      }
    },
    {
      title: '操作',
      valueType: 'option',
      width: '20%',
      render: (text, record, index, action) => {
        if (record.isDefault) {
          return null
        }
        return [
          <a
            key="editable"
            onClick={() => {
              action?.startEditable?.(record.id)
            }}>
            编辑
          </a>,
          <a
            key="delete"
            onClick={() => {
              setDataSource(dataSource.filter(item => item.id !== record.id))
            }}>
            删除
          </a>
        ]
      }
    }
  ]
  const { run: tryGetList } = useRequest(getBusinessgroupList, {
    onSuccess: (result: API.BusinessgroupItem[]) => {
      setState({ ...state, dataSource: result })
    }
  })

  const { run: tryAddBusinessgroup } = useRequest(addBusinessgroup, {
    manual: true,
    onSuccess: async () => {
      tryGetList()
      formRef.current?.resetFields()
      message.success('添加成功')
    },
    onError: e => {
      message.error(e.message)
    }
  })
  const { run: tryUpdateBusinessGroup } = useRequest(updateBusinessGroup, {
    manual: true,
    onSuccess: async () => {
      tryGetList()
      formRef.current?.resetFields()
      message.success('添加成功')
    },
    onError: e => {
      message.error(e.message)
    }
  })

  const { run: tryDelBusinessGroup } = useRequest(delBusinessGroupById, {
    manual: true,
    onSuccess: async () => {
      await tryGetList()
    },
    onError: e => {
      message.error(e.message)
    }
  })

  const { run: tryChangeBusinessGroupStatus } = useRequest(changeBusinessGroupStatus, {
    manual: true,
    onSuccess: async () => {
      await tryGetList()
    },
    onError: e => {
      message.error(e.message)
    }
  })

  const { run: tryGetClientSourceList } = useRequest(getClientSourceList, {
    manual: true,
    onSuccess: result => {
      setState({ ...state, clientSourceList: result })
    }
  })

  const { run: tryAddClientSource } = useRequest(addClientSource, {
    manual: true,
    onSuccess: async () => {
      tryGetClientSourceList()
      formRef.current?.resetFields()
      message.success('添加成功')
    },
    onError: e => {
      message.error(e.message)
    }
  })
  const { run: tryUpdateClientSource } = useRequest(updateClientSource, {
    manual: true,
    onSuccess: async () => {
      tryGetClientSourceList()
      formRef.current?.resetFields()
      message.success('编辑成功')
    },
    onError: e => {
      message.error(e.message)
    }
  })

  useEffect(() => {
    if (!department.length) {
      fetchDepartment()
    }
    if (state.menuId === 2) {
      tryGetClientSourceList()
    }
  }, [state.menuId])
  const mainColumns: ColumnsType<API.BusinessgroupItem> = [
    {
      dataIndex: 'name',
      title: '组名称'
    },
    {
      dataIndex: 'departmentName',
      title: '应用部门'
    },
    {
      dataIndex: 'createTime',
      title: '创建时间'
    },
    {
      dataIndex: 'createName',
      title: '创建人'
    },
    {
      dataIndex: 'activation',
      title: '状态',
      render: text => <span className={!text ? 'text-red-500' : ''}>{text ? '启用' : '停用'}</span>
    },
    {
      dataIndex: 'opreate',
      title: '操作',
      render: (_, record) => (
        <div className="divide-x divide-bg-gray-400 flex flex-row">
          <div
            className="pr-2 text-primary cursor-pointer hover:text-hex-967bbd"
            onClick={() => {
              formRef.current?.setFieldsValue({ ...record })
              setState({ ...state, visible: true, type: modalType.UPDATE })
            }}>
            编辑
          </div>
          {record.activation ? (
            <Popconfirm
              title="是否停用此商机组"
              onConfirm={() =>
                tryChangeBusinessGroupStatus({
                  id: record.id,
                  activation: Number(!record.activation)
                })
              }>
              <div className="px-2 text-red-500 cursor-pointer hover:text-red-600">停用</div>
            </Popconfirm>
          ) : null}
          {!record.activation && (
            <div
              className="px-2 text-primary cursor-pointer hover:text-hex-967bbd"
              onClick={() =>
                tryChangeBusinessGroupStatus({
                  id: record.id,
                  activation: Number(!record.activation)
                })
              }>
              启用
            </div>
          )}

          <Popconfirm title="确认删除？" onConfirm={() => tryDelBusinessGroup(record.id)}>
            <div className="pl-2 text-hex-fd3995 cursor-pointer hover:text-hex-e7026e">
              <Delete />
            </div>
          </Popconfirm>
        </div>
      )
    }
  ]
  const clientSource: ColumnsType<API.ClientSourceList> = [
    {
      dataIndex: 'name',
      title: '来源名称'
    },
    {
      dataIndex: 'opreate',
      title: '操作',
      render: (_, record) => (
        <div className="divide-x divide-bg-gray-400 flex flex-row">
          <div
            className="pr-2 text-primary cursor-pointer hover:text-hex-967bbd"
            onClick={() => {
              formRef.current?.setFieldsValue({ ...record })
              setState({ ...state, visible: true, type: modalType.UPDATE })
            }}>
            编辑
          </div>
        </div>
      )
    }
  ]
  return (
    <PageContainer title={false} breadcrumb={false}>
      <div className="h-full w-full flex flex-row">
        <ShowTitleMenu
          onSelect={onSelect}
          options={[
            { label: '商机状态组', value: 1 },
            { label: '来源设置', value: 2 }
          ]}
          defaultValue={1}
        />
        <div className="w-max-3/4">
          <div className="ml-8 bg-white p-4 shadow-md shadow-hex-3e2c5a relative">
            {state.menuId === 1 ? (
              <>
                <div className="pb-3 border-b border-b-black border-opacity-8 mb-3 flex items-center justify-between">
                  <div>商机状态组</div>
                  <Button
                    type="primary"
                    onClick={() => setState({ ...state, visible: true, type: modalType.CREATE })}>
                    添加新组
                  </Button>
                </div>
                <Table
                  columns={mainColumns}
                  dataSource={state.dataSource}
                  bordered
                  rowKey={row => row.id}
                />
              </>
            ) : null}
            {state.menuId === 2 ? (
              <>
                <div className="pb-3 border-b border-b-black border-opacity-8 mb-3 flex items-center justify-between">
                  <div>来源设置</div>
                  <Button
                    type="primary"
                    onClick={() => setState({ ...state, visible: true, type: modalType.CREATE })}>
                    添加来源
                  </Button>
                </div>
                <Table
                  columns={clientSource}
                  dataSource={state.clientSourceList}
                  bordered
                  rowKey={row => row.id}
                />
              </>
            ) : null}
          </div>
        </div>
        <ModalForm
          visible={state.visible}
          formRef={formRef}
          onVisibleChange={show => setState({ ...state, visible: show })}
          title={`${state.type === modalType.CREATE ? '创建' : '更新'}${
            modalTitleType[state.menuId]
          }`}
          onFinish={async value => {
            try {
              if (state.type === modalType.CREATE && state.menuId === 1) {
                await tryAddBusinessgroup({
                  ...value,
                  stageSettings: JSON.stringify(value.stageSettings.filter(item => !item.isDefault))
                })
              }
              if (state.type === modalType.UPDATE && state.menuId === 1) {
                await tryUpdateBusinessGroup({
                  ...value,
                  stageSettings: JSON.stringify(value.stageSettings.filter(item => !item.isDefault))
                })
              }
              if (state.type === modalType.CREATE && state.menuId === 2) {
                await tryAddClientSource({
                  ...value
                })
              }
              if (state.type === modalType.UPDATE && state.menuId === 2) {
                await tryUpdateClientSource({
                  ...value
                })
              }
              return true
            } catch (error) {
              message.error(`${state.type === modalType.CREATE ? '创建' : '更新'}失败，请重试`)
              return false
            }
          }}>
          {state.menuId === 1 ? (
            <>
              <ProFormText name="name" label="组名称" required />
              <ProFormText name="id" hidden />
              <ProForm.Item name="departmentIds" label="应用部门" required>
                <TreeSelect treeData={department} multiple={true} />
              </ProForm.Item>
              <ProForm.Item
                label="状态设置"
                name="stageSettings"
                initialValue={defaultData}
                trigger="onValuesChange">
                <EditableProTable<StageSettingsType>
                  rowKey="id"
                  toolBarRender={false}
                  columns={columns}
                  recordCreatorProps={{
                    newRecordType: 'dataSource',
                    position: 'bottom',
                    record: () => ({
                      id: Date.now()
                    })
                  }}
                  editable={{
                    type: 'multiple',
                    editableKeys,
                    onChange: setEditableRowKeys,
                    actionRender: (row, _, dom) => {
                      return [dom.delete]
                    }
                  }}
                />
              </ProForm.Item>
            </>
          ) : null}
          {state.menuId === 2 ? (
            <>
              <ProFormText name="id" hidden />
              <ProFormText name="name" label="来源名称" required />
            </>
          ) : null}
        </ModalForm>
      </div>
    </PageContainer>
  )
}

export default Contact
